git-tar-tree: no more void pointer arithmetic
[git.git] / git-am.sh
index 98a390a..4232e27 100755 (executable)
--- a/git-am.sh
+++ b/git-am.sh
@@ -1,20 +1,43 @@
 #!/bin/sh
 #
-#
-. git-sh-setup || die "Not a git archive"
+# Copyright (c) 2005, 2006 Junio C Hamano
 
-usage () {
-    echo >&2 "usage: $0 [--signoff] [--dotest=<dir>] [--utf8] [--3way] <mbox>"
-    echo >&2 " or, when resuming"
-    echo >&2 " $0 [--skip | --resolved]"
-    exit 1;
-}
+USAGE='[--signoff] [--dotest=<dir>] [--utf8] [--binary] [--3way]
+  [--interactive] [--whitespace=<option>] <mbox>...
+  or, when resuming [--skip | --resolved]'
+. git-sh-setup
+
+git var GIT_COMMITTER_IDENT >/dev/null || exit
 
 stop_here () {
     echo "$1" >"$dotest/next"
     exit 1
 }
 
+stop_here_user_resolve () {
+    if [ -n "$resolvemsg" ]; then
+           echo "$resolvemsg"
+           stop_here $1
+    fi
+    cmdline=$(basename $0)
+    if test '' != "$interactive"
+    then
+        cmdline="$cmdline -i"
+    fi
+    if test '' != "$threeway"
+    then
+        cmdline="$cmdline -3"
+    fi
+    if test '.dotest' != "$dotest"
+    then
+        cmdline="$cmdline -d=$dotest"
+    fi
+    echo "When you have resolved this problem run \"$cmdline --resolved\"."
+    echo "If you would prefer to skip this patch, instead run \"$cmdline --skip\"."
+
+    stop_here $1
+}
+
 go_next () {
        rm -f "$dotest/$msgnum" "$dotest/msg" "$dotest/msg-clean" \
                "$dotest/patch" "$dotest/info"
@@ -36,46 +59,12 @@ fall_back_3way () {
        GIT_INDEX_FILE="$dotest/patch-merge-tmp-index" \
        git-write-tree >"$dotest/patch-merge-base+" &&
        # index has the base tree now.
-       (
-           cd "$dotest/patch-merge-tmp-dir" &&
-           GIT_INDEX_FILE="../patch-merge-tmp-index" \
-           GIT_OBJECT_DIRECTORY="$O_OBJECT" \
-           git-apply --index <../patch
-        )
+       GIT_INDEX_FILE="$dotest/patch-merge-tmp-index" \
+       git-apply $binary --cached <"$dotest/patch"
     then
        echo Using index info to reconstruct a base tree...
        mv "$dotest/patch-merge-base+" "$dotest/patch-merge-base"
        mv "$dotest/patch-merge-tmp-index" "$dotest/patch-merge-index"
-    else
-       # Otherwise, try nearby trees that can be used to apply the
-       # patch.
-       (
-           N=10
-
-           # Hoping the patch is against our recent commits...
-           git-rev-list --max-count=$N HEAD
-
-           # or hoping the patch is against known tags...
-           git-ls-remote --tags .
-       ) |
-       while read base junk
-       do
-           # See if we have it as a tree...
-           git-cat-file tree "$base" >/dev/null 2>&1 || continue
-
-           rm -fr "$dotest"/patch-merge-* &&
-           mkdir "$dotest/patch-merge-tmp-dir" || break
-           (
-               cd "$dotest/patch-merge-tmp-dir" &&
-               GIT_INDEX_FILE=../patch-merge-tmp-index &&
-               GIT_OBJECT_DIRECTORY="$O_OBJECT" &&
-               export GIT_INDEX_FILE GIT_OBJECT_DIRECTORY &&
-               git-read-tree "$base" &&
-               git-apply --index &&
-               mv ../patch-merge-tmp-index ../patch-merge-index &&
-               echo "$base" >../patch-merge-base
-           ) <"$dotest/patch"  2>/dev/null && break
-       done
     fi
 
     test -f "$dotest/patch-merge-index" &&
@@ -92,13 +81,17 @@ fall_back_3way () {
     # saying that we reverted all those changes.
 
     git-merge-resolve $orig_tree -- HEAD $his_tree || {
+           if test -d "$GIT_DIR/rr-cache"
+           then
+               git-rerere
+           fi
            echo Failed to merge in the changes.
            exit 1
     }
 }
 
 prec=4
-dotest=.dotest sign= utf8= keep= skip= interactive= resolved=
+dotest=.dotest sign= utf8= keep= skip= interactive= resolved= binary= ws= resolvemsg=
 
 while case "$#" in 0) break;; esac
 do
@@ -113,6 +106,9 @@ do
        --interacti|--interactiv|--interactive)
        interactive=t; shift ;;
 
+       -b|--b|--bi|--bin|--bina|--binar|--binary)
+       binary=t; shift ;;
+
        -3|--3|--3w|--3wa|--3way)
        threeway=t; shift ;;
        -s|--s|--si|--sig|--sign|--signo|--signof|--signoff)
@@ -128,6 +124,12 @@ do
        --sk|--ski|--skip)
        skip=t; shift ;;
 
+       --whitespace=*)
+       ws=$1; shift ;;
+
+       --resolvemsg=*)
+       resolvemsg=$(echo "$1" | sed -e "s/^--resolvemsg=//"); shift ;;
+
        --)
        shift; break ;;
        -*)
@@ -156,22 +158,21 @@ then
 else
        # Make sure we are not given --skip nor --resolved
        test ",$skip,$resolved," = ,,, ||
-               die "we are not resuming."
+               die "Resolve operation not in progress, we are not resuming."
 
        # Start afresh.
        mkdir -p "$dotest" || exit
 
-       # cat does the right thing for us, including '-' to mean
-       # standard input.
-       cat "$@" |
-       git-mailsplit -d$prec "$dotest/" >"$dotest/last" || {
+       git-mailsplit -d"$prec" -o"$dotest" -b -- "$@" > "$dotest/last" ||  {
                rm -fr "$dotest"
                exit 1
        }
 
# -s, -u and -k flags are kept for the resuming session after
# a patch failure.
        # -3 and -i can and must be given when resuming.
+       echo "$binary" >"$dotest/binary"
+       echo " $ws" >"$dotest/whitespace"
        echo "$sign" >"$dotest/sign"
        echo "$utf8" >"$dotest/utf8"
        echo "$keep" >"$dotest/keep"
@@ -187,6 +188,10 @@ case "$resolved" in
        fi
 esac
 
+if test "$(cat "$dotest/binary")" = t
+then
+       binary=--allow-binary-replacement
+fi
 if test "$(cat "$dotest/utf8")" = t
 then
        utf8=-u
@@ -195,6 +200,7 @@ if test "$(cat "$dotest/keep")" = t
 then
        keep=-k
 fi
+ws=`cat "$dotest/whitespace"`
 if test "$(cat "$dotest/sign")" = t
 then
        SIGNOFF=`git-var GIT_COMMITTER_IDENT | sed -e '
@@ -210,6 +216,7 @@ this=`cat "$dotest/next"`
 if test "$skip" = t
 then
        this=`expr "$this" + 1`
+       resume=
 fi
 
 if test "$this" -gt "$last"
@@ -224,6 +231,7 @@ do
        msgnum=`printf "%0${prec}d" $this`
        next=`expr "$this" + 1`
        test -f "$dotest/$msgnum" || {
+               resume=
                go_next
                continue
        }
@@ -248,6 +256,13 @@ do
        GIT_AUTHOR_NAME="$(sed -n '/^Author/ s/Author: //p' "$dotest/info")"
        GIT_AUTHOR_EMAIL="$(sed -n '/^Email/ s/Email: //p' "$dotest/info")"
        GIT_AUTHOR_DATE="$(sed -n '/^Date/ s/Date: //p' "$dotest/info")"
+
+       if test -z "$GIT_AUTHOR_EMAIL"
+       then
+               echo "Patch does not have a valid e-mail address."
+               stop_here $this
+       fi
+
        export GIT_AUTHOR_NAME GIT_AUTHOR_EMAIL GIT_AUTHOR_DATE
 
        SUBJECT="$(sed -n '/^Subject/ s/Subject: //p' "$dotest/info")"
@@ -284,7 +299,7 @@ do
            } >"$dotest/final-commit"
            ;;
        *)
-               case "$resolved,$interactive" in
+               case "$resolved$interactive" in
                tt)
                        # This is used only for interactive view option.
                        git-diff-index -p --cached HEAD >"$dotest/patch"
@@ -304,7 +319,7 @@ do
                echo "--------------------------"
                cat "$dotest/final-commit"
                echo "--------------------------"
-               echo -n "Apply? [y]es/[n]o/[e]dit/[v]iew patch/[a]ccept all "
+               printf "Apply? [y]es/[n]o/[e]dit/[v]iew patch/[a]ccept all "
                read reply
                case "$reply" in
                [yY]*) action=yes ;;
@@ -339,7 +354,7 @@ do
 
        case "$resolved" in
        '')
-               git-apply --index "$dotest/patch"
+               git-apply $binary --index $ws "$dotest/patch"
                apply_status=$?
                ;;
        t)
@@ -348,6 +363,19 @@ do
                # trust what the user has in the index file and the
                # working tree.
                resolved=
+               changed="$(git-diff-index --cached --name-only HEAD)"
+               if test '' = "$changed"
+               then
+                       echo "No changes - did you forget update-index?"
+                       stop_here_user_resolve $this
+               fi
+               unmerged=$(git-ls-files -u)
+               if test -n "$unmerged"
+               then
+                       echo "You still have unmerged paths in your index"
+                       echo "did you forget update-index?"
+                       stop_here_user_resolve $this
+               fi
                apply_status=0
                ;;
        esac
@@ -358,7 +386,7 @@ do
                then
                    # Applying the patch to an earlier tree and merging the
                    # result may have produced the same tree as ours.
-                   changed="$(git-diff-index --cached --name-only -z HEAD)"
+                   changed="$(git-diff-index --cached --name-only HEAD)"
                    if test '' = "$changed"
                    then
                            echo No changes -- Patch already applied.
@@ -372,7 +400,7 @@ do
        if test $apply_status != 0
        then
                echo Patch failed at $msgnum.
-               stop_here $this
+               stop_here_user_resolve $this
        fi
 
        if test -x "$GIT_DIR"/hooks/pre-applypatch
@@ -385,7 +413,7 @@ do
        parent=$(git-rev-parse --verify HEAD) &&
        commit=$(git-commit-tree $tree -p $parent <"$dotest/final-commit") &&
        echo Committed: $commit &&
-       git-update-ref HEAD $commit $parent ||
+       git-update-ref -m "am: $SUBJECT" HEAD $commit $parent ||
        stop_here $this
 
        if test -x "$GIT_DIR"/hooks/post-applypatch
@@ -156,22 +158,21 @@ then
        }
 
-       # -s, -u and -k flags are kept for the resuming session after
-       # a patch failure.
+       # -b, -s, -u, -k and --whitespace flags are kept for the
+       # resuming session after a patch failure.
        # -3 and -i can and must be given when resuming.